<?php if( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
/**
 * Plugin Name: Supr3.me Public Preview Link
 * Plugin URI: http://www.supr3.me/plugin/supr3me-public-preview/
 * Description: Supr3.me Public Preview generates preview link for post types that support 'public-preview'
 * Version: 1.0
 * Author: Supr3.me
 * Author URI: http://www.supr3.me
 * Text Domain: spp
 */

// load in our abstract class
if(!class_exists('Supr3meBaseAbstract')) {
  require(__DIR__ . '/lib/Supr3meBaseAbstract.php');
}

class Supr3me_Public_Preview extends Supr3meBaseAbstract {

  /**
   * Constructor method
   * @return void
   */
  public function __construct () {

    // build our parent
    parent::__construct('public-preview', array ());

    // bind to WordPress init
    $this->action('init', 'wordpress_init');
  }


  /**
   * Generates a unique hash to prevent discovering public links
   * @param  int $post_id The ID of the post or null
   * @return string       Returns the generated unique hash
   */
  private function _generate_unique_hash ($post_id = null) {

    // determine the post ID
    if(is_null($post_id)) {
      $post_id = get_the_id();
    }

    // generate our secure hash
    $unique_hash = md5(wp_salt() . $post_id . wp_salt('nonce'));
    $unique_hash = substr($unique_hash, 0, 8);

    // generate our unique hash
    return $unique_hash;
  }

  /**
   * Triggered on the WordPress init method
   * @return void
   */
  public function wordpress_init () {

    // add public preview support to posts
    add_post_type_support('post', 'public-preview');

    // add our query vars
    $this->filter('query_vars',                       'filter_add_query_var');
    $this->filter('wpseo_whitelist_permalink_vars',   'filter_add_query_var');

    // generate our public preview box
    $this->action('post_submitbox_misc_actions',      'action_submitbox_actions');

    // now make our preview available
    $this->action('pre_get_posts',                    'action_pre_get_posts');

  }

  /**
   * Register our query variables
   * @param  array $vars Array of existing vars
   * @return array       Modified array of vars
   */
  public function filter_add_query_var ($vars) {

    // store our two params
    $vars[] = 'pv';
    $vars[] = 'pvh';

    // return our vars
    return $vars;
  }

  /**
   * Generate what to display on our submitbox
   * @return void
   */
  public function action_submitbox_actions () {

    // make sure post supports it and its in the correct status
    if(!post_type_supports(get_post_type(), 'public-preview')) return;
    if(!in_array(get_post_status(), array ('draft', 'pending'))) return;

    // generate our secure hash
    $unique_hash = $this->_generate_unique_hash(get_the_id());

    // generate our preview link
    $preview_link = array ('preview' => 1, 'pv' => get_the_id(), 'pvh' => $unique_hash);
    $preview_link = add_query_arg($preview_link, get_permalink());
    $preview_link = wp_make_link_relative($preview_link);
    $preview_link = sprintf('%s%s', home_url(), $preview_link);

    // load our view
    $this->load_view('postbox_actions', array (
      'preview_link'    => $preview_link,
    ));
  }

  /**
   * Before getting posts, check if we are doing a preview
   * @param  WP_Query $query WP_Query object
   * @return WP_Query
   */
  public function action_pre_get_posts ($query) {

    if(

      // make sure we have a preview link param
      isset($_REQUEST['pv'])

      // check our preview hash exists
      && isset($_REQUEST['pvh'])

      // make sure our preview link param is numeric
      && is_numeric($_REQUEST['pv'])

      // make sure we are on a signel post
      && is_single()

      // make sure our post type supports public preview
      && post_type_supports(get_post_type($_REQUEST['pv']), 'public-preview')

      // make sure our post type is within a valid status
      && in_array(get_post_status($_REQUEST['pv']), array ('draft', 'pending', 'publish'))

      // verify that our public preview hash is correct
      && $_REQUEST['pvh'] === $this->_generate_unique_hash($_REQUEST['pv'])
    ) {

      // now run our filter to manipulate our post results
      $this->filter('posts_results', 'filter_manipulate_posts_results');

    }

    // return our query
    return $query;
  }


  /**
   * Manipulate the post results to mark them as publish
   * @param  array $posts Array of posts we are loading - should only be one
   * @return array        Modified array of posts
   */
  public function filter_manipulate_posts_results ($posts) {

    // this condition shouldn't happen
    if(empty($posts)) return;

    // loop through
    foreach($posts as $key => $post) {

      // if the post is already published, redirect
      if(get_post_status($_REQUEST['pv']) == 'publish') {
        wp_redirect(get_permalink($_REQUEST['pv']), 302);
        exit;
      }

      // set our post to be published
      $posts[$key]->post_status = 'publish';
    }

    // disable comments and pings for this post
    add_filter('comments_open',   '__return_false');
    add_filter('pings_open',      '__return_false');

    // return our updated posts
    return $posts;
  }
}

// instatiate our class
new Supr3me_Public_Preview;